package com.ruoyi.web.controller.file;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SnowFlake;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.file.domain.FilesProject;
import com.ruoyi.file.domain.FilesStorage;
import com.ruoyi.file.domain.FilesTopic;
import com.ruoyi.file.service.IFilesProjectService;
import com.ruoyi.file.service.IFilesStorageService;
import com.ruoyi.file.service.IFilesTopicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 项目管理Controller
 *
 * @author cmc
 * @date 2022-09-05
 */
@RestController
@RequestMapping("/file/project")
public class FilesProjectController extends BaseController {
    @Autowired
    private IFilesProjectService filesProjectService;

    @Autowired
    private IFilesTopicService filesTopicService;

    @Autowired
    private IFilesStorageService filesStorageService;

    @Value("${cmc.profile}")
    private String profile;

    /**
     * 查询项目管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(FilesProject filesProject) {
        List<FilesProject> list = filesProjectService.selectFilesProjectList(filesProject);
        if (filesProject.getStorageAddress() != null && !filesProject.getStorageAddress().equals("")) {
            FilesStorage filesStorage = new FilesStorage();
            filesStorage.setStorageAddress(filesProject.getStorageAddress());
            if (filesStorageService.selectFilesStorageList(filesStorage).size() > 0)
                filesProject.setStorageId(filesStorageService.selectFilesStorageList(filesStorage).get(0).getId());
        }
        if (filesProject.getTopicName() != null && !filesProject.getTopicName().equals("")) {
            FilesTopic filesTopic = new FilesTopic();
            filesTopic.setTopicName(filesProject.getTopicName());
            if (filesTopicService.selectFilesTopicList(filesTopic).size() > 0)
                filesProject.setTopicId(filesTopicService.selectFilesTopicList(filesTopic).get(0).getId());
        }
        return getDataTable(list);
    }

    /**
     * 导出项目管理列表
     */
    @Log(title = "项目管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, FilesProject filesProject) {
        List<FilesProject> list = filesProjectService.selectFilesProjectList(filesProject);
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.exportExcel(response, list, "项目管理数据");
    }

    /**
     * 导出选中项目管理列表
     */
    @Log(title = "成果表", businessType = BusinessType.EXPORT)
    @PostMapping("/exportBatch/{ids}")
    public void exportBatch(HttpServletResponse response, FilesProject filesProject, @PathVariable("ids") String[] ids)
    {
        List<FilesProject> list = new ArrayList<>();
        for (String projectId : ids)
            list.add(filesProjectService.selectFilesProjectById(projectId));
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.exportExcel(response, list, "成果表数据");
    }

    /**
     * 下载项目表模版
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.importTemplateExcel(response, "项目表模板");
    }

    /**
     * 获取项目管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success(filesProjectService.selectFilesProjectById(id));
    }

    /**
     * 获取专题对应项目列表
     */
    @GetMapping("/topicName")
    public AjaxResult getProjectByTopic(String topicName) {
        FilesTopic filesTopic = new FilesTopic();
        filesTopic.setTopicName(topicName);
        String topicId = filesTopicService.selectFilesTopicList(filesTopic).get(0).getId();
        FilesProject filesProject = new FilesProject();
        filesProject.setTopicId(topicId);
        return AjaxResult.success(filesProjectService.selectFilesProjectList(filesProject));
    }

    /**
     * 上传项目excel文件
     */
    @PostMapping("/uploadSheet")
    public AjaxResult uploadProjectSheet(MultipartFile file) throws Exception {
        if (file.isEmpty()) {
            return AjaxResult.error("文件内容为空！");
        } else {
            ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
            List<FilesProject> filesProjectList = util.importExcel(file.getInputStream());
            List<String> IdList = new ArrayList<>();
            for (FilesProject fileProject :  filesProjectService.selectFilesProjectList(new FilesProject())) {
                IdList.add(fileProject.getId());
            }
            for (FilesProject fileProject : filesProjectList)
            {
                if (fileProject.getId() == null || !IdList.contains(fileProject.getId()))
                    fileProject.setId(new SnowFlake().generateId());
                if (IdList.contains(fileProject.getId()))
                    filesProjectService.updateFilesProject(fileProject);
                else
                    filesProjectService.insertFilesProject(fileProject);
            }
            return AjaxResult.success("上传成功");
        }
    }

    /**
     * 新增项目管理
     */
    @Log(title = "项目管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(FilesProject filesProject, @RequestParam(value = "kmlfile",required = false) MultipartFile uploadProjectKmlFile, @RequestParam(value = "cpfile",required = false) MultipartFile uploadCpKmlFile) throws IOException {
        ConvertToId(filesProject);
        filesProject.setId(new SnowFlake().generateId());
        if (uploadProjectKmlFile(filesProject, uploadProjectKmlFile) )
            return AjaxResult.error("项目范围kml文件内容为空");
        if (uploadCpKmlFile(filesProject, uploadCpKmlFile))
            return AjaxResult.error("项目控制点kml文件内容为空");
        return toAjax(filesProjectService.insertFilesProject(filesProject));
    }

    private boolean uploadProjectKmlFile(FilesProject filesProject, MultipartFile uploadProjectKmlFile) throws IOException {
        if (uploadProjectKmlFile != null) {
            if (!uploadProjectKmlFile.isEmpty()) {
                File profilePath = new File(profile);
                if (!profilePath.exists())//若此目录不存在，则创建之
                    profilePath.mkdirs();
                File transferFile = new File( profilePath + "\\" + uploadProjectKmlFile.getOriginalFilename());
                if (!transferFile.exists())
                    uploadProjectKmlFile.transferTo(transferFile);
                filesProject.setProjectKml(transferFile.getName());
            }
            else
                return true;
        }
        return false;
    }

    private boolean uploadCpKmlFile(FilesProject filesProject, MultipartFile uploadCpKmlFile) throws IOException {
        if (uploadCpKmlFile != null) {
            if (!uploadCpKmlFile.isEmpty()) {
                File profilePath = new File(profile);
                if (!profilePath.exists())//若此目录不存在，则创建之
                    profilePath.mkdirs();
                File transferFile = new File( profilePath + "\\" + uploadCpKmlFile.getOriginalFilename());
                if (!transferFile.exists())
                    uploadCpKmlFile.transferTo(transferFile);
                filesProject.setProjectCp(transferFile.getName());
            }
            else
                return true;
        }
        return false;
    }

    private void ConvertToId(FilesProject filesProject) {
        FilesStorage storage = new FilesStorage();
        storage.setStorageAddress(filesProject.getStorageAddress());
        if (filesProject.getStorageAddress() != null && !filesProject.getStorageAddress().equals("")) {
            filesProject.setStorageId(new SnowFlake().generateId());
            if (filesStorageService.selectFilesStorageListE(storage).size() > 0)
                filesProject.setStorageId(filesStorageService.selectFilesStorageListE(storage).get(0).getId());
            else {
                FilesStorage filesStorage = new FilesStorage();
                filesStorage.setId(filesProject.getStorageId());
                filesStorage.setStorageAddress(filesProject.getStorageAddress());
                filesStorage.setAdministrator(getLoginUser().getUsername());
                filesStorage.setRemark("");
                filesStorageService.insertFilesStorage(filesStorage);
            }
        }
        FilesTopic topic = new FilesTopic();
        topic.setTopicName(filesProject.getTopicName());
        if (filesProject.getTopicName() != null && !filesProject.getTopicName().equals("")) {
            filesProject.setTopicId(new SnowFlake().generateId());
            if (filesTopicService.selectFilesTopicList(topic).size() > 0)
                filesProject.setTopicId(filesTopicService.selectFilesTopicList(topic).get(0).getId());
            else {
                FilesTopic filesTopic = new FilesTopic();
                filesTopic.setId(filesProject.getTopicId());
                filesTopic.setTopicName(filesProject.getTopicName());
                filesTopicService.insertFilesTopic(filesTopic);
            }
        }
    }

    /**
     * 修改项目管理
     */
    @Log(title = "项目管理", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(FilesProject filesProject, @RequestParam(value = "kmlfile",required = false) MultipartFile uploadProjectKmlFile, @RequestParam(value = "cpfile",required = false) MultipartFile uploadCpKmlFile) throws IOException {
        ConvertToId(filesProject);
        if (uploadProjectKmlFile(filesProject, uploadProjectKmlFile) )
            return AjaxResult.error("项目范围kml文件内容为空");
        if (uploadCpKmlFile(filesProject, uploadCpKmlFile))
            return AjaxResult.error("项目控制点kml文件内容为空");
        return toAjax(filesProjectService.updateFilesProject(filesProject));
    }

    /**
     * 删除项目管理
     */
    @Log(title = "项目管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(filesProjectService.deleteFilesProjectByIds(ids));
    }
    /**
     * 根据项目编号查询项目信息
     */
    @GetMapping("/query/{projectNumbers}")
    public AjaxResult getInfoByProjectNumber(@PathVariable String[] projectNumbers){
        return AjaxResult.success(filesProjectService.selectFilesProjectByNumbers(projectNumbers));
    }
}